window.Gleam.initSearch([{"doc":"form_coder","title":"form_coder","content":"# `form_coder`\n\nA Gleam library to encode and decode `x-www-form-urlencoded` data.\n\nThis data is a superset of your traditional GET query parameter format;\ne.g. `foo=bar&amp;baz=qux`. While there&#39;s no established standard for conveying data\nstructures such as lists and maps, `form_coder` follows the format used by both\nRails and Plug. Namely:\n\n- A list like `[#(&quot;foo&quot;, [&quot;bar&quot;, &quot;baz&quot;])]` is encoded to `foo[]=bar&amp;foo[]=baz`\n- A map like `[#(&quot;foo&quot;, %{&quot;bar&quot; =&gt; &quot;qux&quot;})]` (map in Elixir syntax for brevity) is\n  encoded as `foo[bar]=qux`.\n\n## Examples\n\n### Encoding\n\n```gleam\n&gt; form_coder.encode([#(&quot;foo&quot;, QStr(&quot;bar&quot;)), #(&quot;baz&quot;, QStr(&quot;qux&quot;))])\n&quot;foo=bar&amp;baz=qux&quot;\n```\n\n```gleam\n&gt; form_coder.encode([#(&quot;user_ids&quot;, QList([QStr(&quot;1&quot;), QStr(&quot;2&quot;)]))])\n&quot;user_ids[]=1&amp;user_ids[]=2&quot;\n```\n\n```gleam\n&gt; form_coder.encode([#(&quot;person&quot;, QMap(dict.from_list([#(&quot;name&quot;, QStr(&quot;Joe&quot;)), #(&quot;age&quot;, QStr(&quot;71&quot;))])))])\n&quot;person[age]=71&amp;person[name]=Joe&quot;\n```\n\n**What&#39;s with `QStr`, `QList`, and `QMap`?**\n\nWell, in the type specification, the argument must take the form\n`List(#(String, _))`, where I can choose what to replace `_` with. Choosing\nany one of `String`, `List`, or `Dict` would make using the other two very\ninconvenient. Using some type variable `a` would enforce that all &quot;values&quot;\nhave the same type (i.e. you can&#39;t have some values as strings and others as\nlists) and `form_coder` would have to know how to encode your arbitrary `a` and\nthat doesn&#39;t seem possible.\n\nSince Gleam doesn&#39;t have union types, I created an algebraic type with variants\nto wrap each supported type; hence `QStr`, `QList`, and `QMap`.\n\n### Decoding\n\n```gleam\n&gt; form_coder.decode(&quot;foo=bar&amp;baz=qux&quot;, dynamic.list(of: dynamic.tuple2(dynamic.string, dynamic.string)))\n[#(&quot;foo&quot;, &quot;bar&quot;), #(&quot;baz&quot;, &quot;qux&quot;)]\n```\n\nSupport for decoding the more sophisticated types (List, Map) is planned, but\nnot yet implemented.\n\n## Quick start\n\n```sh\ngleam build # Compile the project\ngleam test  # Run the tests\ngleam shell # Run an Erlang shell\n```\n\n## Installation\n\n```sh\ngleam add form_coder\n```\n","url":"index.html"},{"doc":"form_coder","title":"DecodeError","content":"pub type DecodeError {\n  InvalidQuery\n  DynamicError(List(dynamic.DecodeError))\n}\n\nInvalidQuery\n\n\nDynamicError(List(dynamic.DecodeError))\n\n\nSynonyms:\nform_coder.DecodeError\nform_coder DecodeError","url":"form_coder.html#DecodeError"},{"doc":"form_coder","title":"Query","content":"pub type Query {\n  QStr(String)\n  QList(List(Query))\n  QMap(Dict(String, Query))\n}\n\nQStr(String)\n\n\nQList(List(Query))\n\n\nQMap(Dict(String, Query))\n\n\nSynonyms:\nform_coder.Query\nform_coder Query","url":"form_coder.html#Query"},{"doc":"form_coder","title":"decode","content":"pub fn decode(\n  from encoded: String,\n  using decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; Result(a, form_coder.DecodeError)\n Decode the URL query string into a data structure as specified by `decoder`\n (likely a function from `gleam/dynamic`).\nSynonyms:\nform_coder.decode\nform_coder decode","url":"form_coder.html#decode"},{"doc":"form_coder","title":"encode","content":"pub fn encode(contents: List(#(String, Query))) -&gt; String\n Encode a list of pairs into a URL query string\n\n The values of these pairs should be of the `Query` type.\n\n ## Examples\n\n    &gt; encode([#(&quot;foo&quot;, QStr(&quot;bar bar&quot;)), #(&quot;baz&quot;, QStr(&quot;qux&quot;))])\n    &quot;foo=bar+bar&amp;baz=qux&quot;\n\n    &gt; encode([#(&quot;foo&quot;, QList([QStr(&quot;bar&quot;), QStr(&quot;b!z&quot;)]))])\n    &quot;foo%5B%5D=bar&amp;foo%5B%5D=b%21z&quot;\n\n    &gt; encode([#(&quot;foo&quot;, QMap(map.from_list([#(&quot;bar&quot;, &quot;baz&quot;)])))])\n    &quot;foo%5Bbar%5D=baz&quot;\n\nSynonyms:\nform_coder.encode\nform_coder encode","url":"form_coder.html#encode"},{"doc":"form_coder","title":"form_coder","content":" x-www-form-urlencoded codec\n\n The codec follows the WHATWG specification available at:\n https://url.spec.whatwg.org/#application/x-www-form-urlencoded but uses\n UTF-8 as the only supported encoding.","url":"form_coder.html"}]);